package solutions

import (
	"database/sql"
	"errors"
	"fmt"
	"github.com/disintegration/imaging"
	"github.com/levigross/grequests"
	"github.com/schollz/progressbar/v3"
	"github.com/xuri/excelize/v2"
	"math"
	"net/http"
	"os"
	"pestiot/eneity"
	"pestiot/static"
	"strings"
	"sync"
	"time"
)

type datImage struct {
	ImageID   int64
	ImageCode string
	OriginUrl sql.NullString
}

func LocalImageFileCheck(provId, beginDate, endDate, fileDir string, threadNum int) (filepath, msg string) {
	fmt.Printf("开始核验T_Dat_Image_%s[%s~%s]的图片\n", provId, beginDate, endDate)

	taskGroup, allCount, err := getDBImageList(provId, beginDate, endDate, threadNum)
	if err != nil {
		fmt.Println("获取数据异常！")
		return "", ""
	}

	wg := sync.WaitGroup{}
	wg.Add(len(taskGroup))
	noImageChan := make(chan []interface{}, allCount)
	fmt.Printf("核验数据量:%d,划分组数:%d\n", allCount, len(taskGroup))
	bar := progressbar.Default(allCount)
	for _, tasks := range taskGroup {
		tasks := tasks
		go func() {
			defer wg.Done()
			for _, task := range tasks {
				originalPath := fmt.Sprintf("%s/%s", fileDir, strings.ReplaceAll(task.ImageCode, "?pa=Y", ""))
				pressedPath := strings.ReplaceAll(originalPath, "original", "pressed")
				noOriginal := false
				noPressed := false
				if _, err := os.Stat(originalPath); err != nil && os.IsNotExist(err) {
					noOriginal = downOriginalFile(originalPath, task)
				}
				if _, err := os.Stat(pressedPath); err != nil && os.IsNotExist(err) {
					noPressed = downPressedFile(pressedPath, task, noOriginal, originalPath)
				}
				if noOriginal || noPressed {
					noImageChan <- []interface{}{task.ImageID, noOriginal, noPressed, task.ImageCode, task.OriginUrl.String}
				}
				bar.Add(1)
			}
		}()
	}
	wg.Wait()
	close(noImageChan)
	return genNoImageDataReport(noImageChan, fmt.Sprintf("%s(%s~%s)", static.Citys[provId], beginDate, endDate))
}

func getDBImageList(provId, beginDate, endDate string, threadNum int) ([][]datImage, int64, error) {
	dbInfo := &eneity.DbInfo{
		Server:   static.Server,
		User:     static.User,
		Password: static.Password,
		Port:     int32(static.Port),
		Database: static.Database,
		Dbtype:   static.Dbtype,
	}
	db, err := dbInfo.GetConnect()
	if err != nil {
		fmt.Println("sql连接异常:", err)
		return nil, 0, err
	}

	sqllan := fmt.Sprintf("select ImageID, ImageCode, originurl from T_Dat_Image_%s WITH(NOLOCK) where DataYMD >= '%s' and DataYMD <= '%s'", provId, beginDate, endDate)
	fmt.Println(sqllan)
	rows, err := db.Query(sqllan)
	if err != nil {
		fmt.Printf("获取(T_Dat_Image_%s[%s~%s])图片数据异常:%s\n", provId, beginDate, endDate, err.Error())
		return nil, 0, err
	} else {
		fmt.Println("获取到数据")
	}
	var datImageList []datImage
	for rows.Next() {
		di := datImage{}
		err := rows.Scan(&di.ImageID, &di.ImageCode, &di.OriginUrl)
		if err != nil {
			fmt.Printf("(T_Dat_Image_%s[%s~%s])数据rows.Scan异常:%s\n", provId, beginDate, endDate, err.Error())
			return nil, 0, err
		}
		datImageList = append(datImageList, di)
	}
	err = db.Close()
	if err != nil {
		fmt.Println("数据库连接关闭异常:", err)
		return nil, 0, err
	}
	batch := len(datImageList) / threadNum
	return splitSlice(datImageList, batch+1), int64(len(datImageList)), nil
}

func downOriginalFile(savePath string, di datImage) bool {
	if di.OriginUrl.Valid {
		err := downRemoteFile(savePath, di.OriginUrl.String)
		if err == nil {
			return true
		}
	}
	err := downRemoteFile(savePath, fmt.Sprintf("https://iotimage.pestiot.com/iotImage/%s", di.ImageCode))
	if err == nil {
		return true
	}
	return false
}

func downPressedFile(savePath string, di datImage, noOriginal bool, originalPath string) bool {
	err := downRemoteFile(
		savePath,
		fmt.Sprintf(
			"https://iotimage.pestiot.com/iotImage/%s",
			strings.ReplaceAll(di.ImageCode, "original", "pressed")))
	if err == nil {
		return true
	}
	if noOriginal {
		return false
	}
	err = genThumbByOriginFile(savePath, originalPath)
	if err == nil {
		return true
	}
	return false
}

func downRemoteFile(savePath, url string) error {
	response, err := grequests.Get(url, &grequests.RequestOptions{
		DialTimeout:    10 * time.Second,
		RequestTimeout: 30 * time.Second,
	})
	if err != nil {
		return errors.New(fmt.Sprintf("[DownloadError]%s", err.Error()))
	}
	if response.StatusCode != http.StatusOK {
		return errors.New(fmt.Sprintf("[DownloadError][%d]%s", response.StatusCode, response.String()))
	}
	err = response.DownloadToFile(savePath)
	return err
}

func genNoImageDataReport(noImageChan chan []interface{}, taskName string) (filepath, msg string) {
	if len(noImageChan) > 0 {
		xlsx := excelize.NewFile()

		header := []interface{}{"ImageID", "原图", "缩略图", "ImageCode", "OriginUrl"}
		err := xlsx.SetSheetRow("Sheet1", "A1", &header)
		idx := 0
		for itm := range noImageChan {
			row := fmt.Sprintf("A%d", idx+2)
			err := xlsx.SetSheetRow("Sheet1", row, &itm)
			if err != nil {
				fmt.Println(fmt.Sprintf("插入行%s异常:", row), err)
			}
			idx++
		}

		xlsxFilePath := fmt.Sprintf("./file/[result]%s图片未下载情况统计结果_%d.xlsx", taskName, time.Now().Unix())
		err = xlsx.SaveAs(xlsxFilePath)
		if err != nil {
			fmt.Println("保存excel异常:", err)
			return "", ""
		}
		msg := fmt.Sprintf("%s图片存在未下载情况", taskName)
		return xlsxFilePath, msg
	} else {
		return "", ""
	}
}

func genThumbByOriginFile(savePath, originalPath string) error {
	width := 400
	height := 400
	// 解码图像
	img, err := imaging.Open(originalPath)
	if err != nil {
		return errors.New(fmt.Sprintf("[OriginImg2ObjectError]%s", err.Error()))
	}

	// 获取图像尺寸
	imgWidth := img.Bounds().Dx()
	imgHeight := img.Bounds().Dy()

	// 计算缩略图宽高
	var thumbWidth, thumbHeight int
	if imgWidth > imgHeight {
		thumbWidth = width
		thumbHeight = int(math.Floor(float64(imgHeight*width) / float64(imgWidth)))
	} else {
		thumbWidth = int(math.Floor(float64(imgWidth*height) / float64(imgHeight)))
		thumbHeight = height
	}

	// 生成缩略图
	thumb := imaging.Resize(img, thumbWidth, thumbHeight, imaging.Lanczos)

	err = imaging.Save(thumb, savePath)
	return err
}

func splitSlice[T any](slice []T, size int) [][]T {
	var result [][]T

	for i := 0; i < len(slice); i += size {
		end := i + size

		if end > len(slice) {
			end = len(slice)
		}

		result = append(result, slice[i:end])
	}

	return result
}
